/*
 * This file is part of the e-field program written by Adam Cornett
 * This program is distrubuted under the General Public License.
 * The license is distrubed with the source in the file 'gpl.txt'
 */
package com.ajc.esim.core;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;

/**
 * Not currently in use. This class represents a 2D rectangular charge. It is
 * not currently implemented due to the complexity of the math required to
 * calculate the force it exerts.
 * 
 * @see sci.core.Charge
 */
public class LineCharge extends Charge {
	private double height = 0;

	private double length = 0;

	public LineCharge(double charge, double x, double y, double w, double l) {
		super(x, y, charge);
		this.setType(Charge.ChargeType.LineCharge);
		this.height = w;
		this.length = l;
	}

	@Override
	public boolean checkClick(MouseEvent e) {
		double mult = 1; // for future use, locations will be stored in
		// meters
		// not in pixels by version 2, mult will be a conversion
		// factor.

		double xLoc = e.getX() * mult;
		double yLoc = e.getY() * mult;

		if (isInside(xLoc, yLoc)) {
			return true;
		}

		return false;
	}

	@Override
	public double getHeight() {
		if (height < 0) {
			height = 0;
		}

		return height;
	}

	@Override
	public double getLength() {
		return length;
	}

	@Override
	public double getSize() {
		return Calc.dist(0, 0, this.length, this.height);
	}

	@Override
	public double getX() {
		return (super.getX() + (this.length / 2));
	}

	@Override
	public double getY() {
		return (super.getY() + (this.height / 2));
	}

	@Override
	public boolean isInside(double x, double y) {
		if ((x > this.getX()) && (x < (this.getX() + this.length))) {
			if ((y > this.getY()) && (y < (this.getY() + this.height))) {
				return true;
			}
		}

		return false;
	}

	@Override
	public void paint(Graphics g) {
		if (this.getCharge() < 0) {
			g.setColor(Launcher.dat.negColor);
		} else {
			g.setColor(Launcher.dat.posColor);
		}

		int l = (int) this.length;
		int w = (int) this.height;
		g.fillRect((int) this.getX(), (int) this.getY(), l, w);

		if (isSel) {
			g.setColor(Color.WHITE);
		} else {
			g.setColor(Color.BLACK);
		}

		g.drawRect((int) this.getX() - 1, (int) this.getY() - 1, l + 2, w + 2);
		g.setColor(Launcher.dat.text);
		g.drawString(this.getChargeText(), ((int) this.getX() + (l / 2)) - 3,
				(int) this.getY() + (w / 2) + 5);
	}

	@Override
	public void reBuildShape() {
		myShape = new Rectangle2D.Double(this.getX(), this.getY(), this
				.getHeight(), this.getLength());
	}

	public void setHeight(double width) {
		this.height = width;
	}

	public void setLength(double length) {
		if (length < 0) {
			length = 0;
		}

		this.length = length;
	}
}
