/*
 * This file is part of the e-field program written by Adam Cornett
 * This program is distrubuted under the General Public License.
 * The license is distrubed with the source in the file 'gpl.txt'
 */
package com.ajc.esim.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.KeyListener;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.ajc.esim.core.Charge;

public class ChargePanel extends JPanel {

	private JTextField chargeField = null;

	private JLabel chargeLabel = null;

	private JLabel xLabel = null;

	private JTextField xLocationField = null;

	private JLabel yLabel = null;

	private JTextField yLocationField = null;

	/**
	 * Creates a new ChargePanel object.
	 */
	public ChargePanel() {
		super();
		initialize();
	}

	/**
	 * Creates a new ChargePanel object.
	 * 
	 * @param c
	 */
	public ChargePanel(Charge c) {
		super();
		initialize();
		this.setCharge(c);
	}

	public double getChargeVal() throws NumberFormatException {
		return Double.parseDouble(chargeField.getText());
	}

	public double getXVal() throws NumberFormatException {
		return Double.parseDouble(xLocationField.getText());
	}

	public double getYVal() throws NumberFormatException {
		return Double.parseDouble(yLocationField.getText());
	}

	public void setCharge(Charge c) {
		this.chargeField.setText("" + c.getCharge());
		this.xLocationField.setText("" + c.getX());
		this.yLocationField.setText("" + c.getY());
	}

	public void setKeyListener(KeyListener k) {
		xLocationField.addKeyListener(k);
		yLocationField.addKeyListener(k);
		chargeField.addKeyListener(k);
	}

	public void setXY(double x, double y) {
		this.xLocationField.setText("" + x);
		this.yLocationField.setText("" + y);
	}

	private JTextField getChargeField() {
		if (chargeField == null) {
			chargeField = new JTextField();
			chargeField.setInputVerifier(new DoubleVerifier());
			chargeField.requestFocus(true);
			chargeField.setToolTipText("Charge, in coulombs, of the charge.  For best results, keep the magnitude between 0.1 and 1e-8 C.");
		}

		return chargeField;
	}

	private JTextField getXLocationField() {
		if (xLocationField == null) {
			xLocationField = new JTextField();
			xLocationField.setInputVerifier(new DoubleVerifier());
			xLocationField.setToolTipText("X Location (in pixels) of the charge.");
		}

		return xLocationField;
	}

	private JTextField getYLocationField() {
		if (yLocationField == null) {
			yLocationField = new JTextField();
			yLocationField.setInputVerifier(new DoubleVerifier());
			yLocationField.setToolTipText("Y Location (in pixels) of the charge.");
		}

		return yLocationField;
	}

	private void initialize() {
		GridBagConstraints yLocationFieldConstraints = new GridBagConstraints();
		yLocationFieldConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		yLocationFieldConstraints.gridy = 2;
		yLocationFieldConstraints.weightx = 1.0;
		yLocationFieldConstraints.gridx = 1;

		GridBagConstraints xLocationFieldConstraints = new GridBagConstraints();
		xLocationFieldConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		xLocationFieldConstraints.gridy = 1;
		xLocationFieldConstraints.weightx = 1.0;
		xLocationFieldConstraints.gridx = 1;

		GridBagConstraints chargeFieldConstraints = new GridBagConstraints();
		chargeFieldConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		chargeFieldConstraints.gridy = 0;
		chargeFieldConstraints.weightx = 1.0;
		chargeFieldConstraints.gridx = 1;

		GridBagConstraints yLabelConstraints = new GridBagConstraints();
		yLabelConstraints.gridx = 0;
		yLabelConstraints.gridy = 2;

		GridBagConstraints xLabelConstraints = new GridBagConstraints();
		xLabelConstraints.gridx = 0;
		xLabelConstraints.gridy = 1;

		GridBagConstraints chargeLabelConstraints = new GridBagConstraints();
		chargeLabelConstraints.gridx = 0;
		chargeLabelConstraints.gridy = 0;

		chargeLabel = new JLabel("Charge: ");
		xLabel = new JLabel("X Location: ");
		yLabel = new JLabel("Y Location: ");

		this.setLayout(new GridBagLayout());
		this.setSize(300, 84);
		this.add(chargeLabel, chargeLabelConstraints);
		this.add(xLabel, xLabelConstraints);
		this.add(yLabel, yLabelConstraints);
		this.add(getChargeField(), chargeFieldConstraints);
		this.add(getXLocationField(), xLocationFieldConstraints);
		this.add(getYLocationField(), yLocationFieldConstraints);
	}
} // @jve:decl-index=0:visual-constraint="10,10"
