/*
 * Copyright Adam Cornett 2006
 */
package com.ajc.esim.gui;

import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.ajc.esim.core.Launcher;

/**
 * DOCUMENT ME!
 * 
 * @since 1.28
 */
public class DrawPHere extends EFDialog {
	private JButton cancelButton = null;

	private JButton okButton = null;

	private JLabel voltLabel = null;

	private JPanel buttonsPanel = null;

	private JPanel inputPanel = null;

	private JPanel jContentPane = null;

	private JTextField voltageField = null;

	public DrawPHere(Frame parent) {
		super(parent);
		initialize();
	}

	public DrawPHere(Frame parent, double val) {
		this(parent);
		voltageField.setText(val + "");
	}

	@Override
	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getActionCommand().equals("OK")) {
			okAction();
		} else if (arg0.getActionCommand().equals("CANCEL")) {
			cancelAction();
		}
	}

	@Override
	protected void cancelAction() {
		this.dispose();
	}

	@Override
	protected void okAction() {
		if (add()) {
			dispose();
		}
	}

	private boolean add() {
		double voltage;

		try {
			voltage = Double.parseDouble(voltageField.getText());
		} catch (NumberFormatException e) {
			JOptionPane.showMessageDialog(this,
					"Error!\nPlease check voltage and try again.");

			return false;
		}

		Launcher.win.peClicks.add(voltage);
		Launcher.win.reDraw();

		return true;
	}

	private JPanel getButtonsPanel() {
		if (buttonsPanel == null) {
			buttonsPanel = new JPanel();
			buttonsPanel.add(getOkButton(), null);
			buttonsPanel.add(getCancelButton(), null);
		}

		return buttonsPanel;
	}

	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText("Cancel");
			cancelButton.setActionCommand("CANCEL");
			cancelButton.addActionListener(this);
		}

		return cancelButton;
	}

	private JPanel getInputPanel() {
		if (inputPanel == null) {
			voltLabel = new JLabel();
			voltLabel.setText("Voltage:");
			inputPanel = new JPanel();
			inputPanel.add(voltLabel, null);
			inputPanel.add(getVoltageField(), null);
		}

		return inputPanel;
	}

	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getButtonsPanel(), java.awt.BorderLayout.SOUTH);
			jContentPane.add(getInputPanel(), java.awt.BorderLayout.CENTER);
		}

		return jContentPane;
	}

	private JButton getOkButton() {
		if (okButton == null) {
			okButton = new JButton();
			okButton.setText("OK");
			okButton.setActionCommand("OK");
			okButton.addActionListener(this);
		}

		return okButton;
	}

	private JTextField getVoltageField() {
		if (voltageField == null) {
			voltageField = new JTextField();
			voltageField.setColumns(24);
			voltageField.setInputVerifier(new DoubleVerifier());
			voltageField.addKeyListener(this);
		}

		return voltageField;
	}

	private void initialize() {
		this.setSize(362, 96);
		this.setTitle("Add Potential Line...");
		this.setContentPane(getJContentPane());
		this.setLocationRelativeTo(null);
	}
} // @jve:decl-index=0:visual-constraint="10,10"
