/*
 * This file is part of the e-field program written by Adam Cornett
 * This program is distrubuted under the General Public License.
 * The license is distrubed with the source in the file 'gpl.txt'
 */
package com.ajc.esim.gui;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JDialog;

/**
 * Abstract class for all Dialogs in this program, requires action handling and
 * implements key listeners for the enter and escape key, has two methods for
 * 'ok' or 'cancel' actions.
 * 
 * @since 1.28
 */
public abstract class EFDialog extends JDialog implements ActionListener,
		KeyListener {

	public EFDialog(Frame Parent) {
		super(Parent);
		this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
	}

	public abstract void actionPerformed(ActionEvent arg0);

	protected abstract void cancelAction();

	public void keyPressed(KeyEvent e) {
		if (e.getKeyCode() == 27) {
			cancelAction();
		} else if (e.getKeyCode() == 10) {
			okAction();
		}
	}

	public void keyReleased(KeyEvent e) {
		if (e.getKeyCode() == 27) {
			cancelAction();
		} else if (e.getKeyCode() == 10) {
			okAction();
		}
	}

	public void keyTyped(KeyEvent e) {
		if (e.getKeyCode() == 27) {
			cancelAction();
		} else if (e.getKeyCode() == 10) {
			okAction();
		}
	}

	protected abstract void okAction();
}
