/*
 * This file is part of the e-field program written by Adam Cornett
 * This program is distrubuted under the General Public License.
 * The license is distrubed with the source in the file 'gpl.txt'
 */
package com.ajc.esim.gui;

import java.awt.event.ActionEvent;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import com.ajc.esim.core.Charge;
import com.ajc.esim.core.Launcher;

/**
 * Edit Charge Dialog
 */
public class EditCharge extends EFDialog {
	private Charge cc = null;

	private ChargePanel cp = null;

	private JButton applyButton = null;

	private JButton cancelButton = null;

	private JButton okButton = null;

	private JPanel panel2 = null;

	/**
	 * Creates a new EditCharge object.
	 */
	public EditCharge(JFrame owner) {
		super(owner);

		if (Launcher.win.currentCharge == null) {
			this.dispose();
		}

		cc = Launcher.win.currentCharge;
		this.cp = new ChargePanel(cc);
		initialize();
	}

	@Override
	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getActionCommand().equals("CANCEL")) {
			cancelAction();
		} else if (arg0.getActionCommand().equals("OK")) {
			okAction();
		} else if (arg0.getActionCommand().equals("APPLY")) {
			if (changeCharge()) {
				changeCharge();
				Launcher.win.updateAll();
			}
		}
	}

	public void initialize() {
		this.setSize(303, 141);
		this.setModal(true);
		cp.setKeyListener(this);
		this.add(cp, java.awt.BorderLayout.NORTH); // Generated
		this.add(getPanel2(), java.awt.BorderLayout.SOUTH); // Generated
		this.addKeyListener(this);
		this.setLocationRelativeTo(Launcher.win);
		this.setTitle("Editing " + Launcher.win.currentCharge);
	}

	@Override
	protected void cancelAction() {
		this.dispose();
	}

	@Override
	protected void okAction() {
		if (changeCharge()) {
			Launcher.win.updateAll();
			this.dispose();
			Launcher.win.EFPanel.repaint();
		}
	}

	private boolean changeCharge() {
		try {
			cc.setCharge(cp.getChargeVal());
			cc.setX(cp.getXVal());
			cc.setY(cp.getYVal());
		} catch (NumberFormatException e) {
			JOptionPane.showMessageDialog(this,
					"Error!\nPlease check fields and try again.");

			return false;
		}

		Launcher.dat.updateScales();

		return true;
	}

	private JButton getApplyButton() {
		if (applyButton == null) {
			applyButton = new JButton("Apply");
			applyButton.setName("Apply JButton");
			applyButton.setActionCommand("APPLY");
			applyButton.addActionListener(this);
		}

		return applyButton;
	}

	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton("Cancel");
			cancelButton.setName("Cancel JButton");
			cancelButton.setActionCommand("CANCEL");
			cancelButton.addActionListener(this);
		}

		return cancelButton;
	}

	private JButton getOkButton() {
		if (okButton == null) {
			okButton = new JButton("OK");
			okButton.setName("OK JButton");
			okButton.setActionCommand("OK");
			okButton.addActionListener(this);
		}

		return okButton;
	}

	private JPanel getPanel2() {
		if (panel2 == null) {
			panel2 = new JPanel();
			panel2.add(getOkButton(), null); // Generated
			panel2.add(getApplyButton(), null); // Generated
			panel2.add(getCancelButton(), null); // Generated
			panel2.addKeyListener(this);
		}

		return panel2;
	}
} // @jve:decl-index=0:visual-constraint="10,10"
