/*
 * This file is part of the e-field program written by Adam Cornett
 * This program is distrubuted under the General Public License.
 * The license is distrubed with the source in the file 'gpl.txt'
 */
package com.ajc.esim.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Toolkit;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.URL;
import java.util.ArrayList;
import java.util.ListIterator;

import javax.swing.*;
import javax.swing.plaf.metal.MetalLookAndFeel;

import com.ajc.esim.core.Calc;
import com.ajc.esim.core.Charge;
import com.ajc.esim.core.Launcher;

/**
 * Main Window for the program.
 */
public class MainWindow extends JFrame implements KeyListener {

	public Color backColor = null;

	public JPanel bottomPanel = null, buttonPanel = null, controls = null,
			mainPanel = null, numLandResNumPanel = null, auxPanel = null,
			paintModePanel = null, statusPanel = null,
			clearAndReDrawPanel = null;

	public Charge currentCharge;

	public EFPanel EFPanel = null;

	public OptionsFrame optwin = null;

	private JList chargeList = null;

	private JButton clearButton = null, editButton = null, reDrawButton = null,
			removeButton = null, addChargeButton = null;

	private JCheckBox fieldLineBox = null, potLinesBox = null;

	private JSlider fieldLineSlider = null, potResSlider = null;

	private URL imageURL = null;

	private JLabel label = null, numLinesLabel = null, forceLabel = null,
			distLabel = null, potentialLabel = null, potRes = null;

	private JScrollPane listScrollPane = null;

	private JMenu menuActions = null, menuFile = null, menuHelp = null;

	private JMenuItem menuActionsAdd = null, menuActionsEdit = null,
			menuActionsRemove = null, menuActionsRemoveAll = null,
			menuFileExit = null, menuFileLoad = null, menuFileOptions = null,
			menuFilePageSetup = null, menuFilePrint = null,
			menuFileSave = null, menuHelpAbout = null;

	private JMenuBar menuBar;

	private MainWinActionHandler mwah = new MainWinActionHandler();

	private JMenuItem popAddLine = null, popAddPoint = null, popDrawPE = null,
			popDrawPE2 = null, popEdit = null, popRemove = null;

	private JPopupMenu popAltMenu = null, popUpMenu = null;

	protected ArrayList<Double> peClicks = new ArrayList<Double>();

	protected int x2;

	protected int y2;

	/**
	 * Creates a new Main object.
	 */
	public MainWindow() {
		super();
		initialize();
		// this.setExtendedState(6);
	}

	public int getEFHeight() {
		return EFPanel.getHeight();
	}

	public int getEFWidth() {
		return EFPanel.getWidth();
	}

	public void keyPressed(KeyEvent e) {
		if (e.getKeyCode() == 127) {
			Launcher.dat.removeCharge(currentCharge);
			this.nextCurrent();
			updateList();
			EFPanel.repaint();
		}
	}

	public void keyReleased(KeyEvent e) {
		if (e.getKeyCode() == 127) {
			Launcher.dat.removeCharge(currentCharge);
			this.nextCurrent();
			updateList();
			EFPanel.repaint();
		}
	}

	public void keyTyped(KeyEvent e) {
		if (e.getKeyCode() == 127) {
			Launcher.dat.removeCharge(currentCharge);
			this.nextCurrent();
			updateList();
			EFPanel.repaint();
		}
	}

	public boolean reDraw() {
		EFPanel.drawOffScreen();
		EFPanel.repaint();

		return true;
	}

	@Override
	public void removeAll() {
		for (int i = 0; i < Launcher.dat.getChargeLength(); i++) {
			Launcher.dat.flushCharges();
		}

		currentCharge = null;
		updateList();
		EFPanel.repaint();
	}

	public void setStatusLabel(String text) {
		label.setText(text);
	}

	public void updateAll() {
		EFPanel.drawOffScreen();
		EFPanel.repaint();
		this.updateLabel();
		this.updateList();
	}

	private void chargeMove(java.awt.event.MouseEvent e) {
		if (currentCharge != null) {
			currentCharge.setX(e.getX());
			currentCharge.setY(e.getY());
			currentCharge.drawOutline(EFPanel.getGraphics());
		}
	}

	private boolean checkSel(java.awt.event.MouseEvent e) {
		boolean found = false;
		for (Charge temp : Launcher.dat.getCharges()) {
			temp.isSel = false;
			if (temp.checkClick(e) && !found) {
				temp.isSel = true;
				chargeList.setSelectedIndex(Launcher.dat.getChargeIndex(temp));
				currentCharge = temp;
				updateLabel();
				found = true;
			}
		}
		return found;
	}

	private JButton getAddChargeButton() {
		if (addChargeButton == null) {
			addChargeButton = new JButton("Add");
			addChargeButton.setActionCommand("ADD");
			addChargeButton.addActionListener(mwah);
		}

		return addChargeButton;
	}

	private JPanel getAuxPanel() {
		if (auxPanel == null) {
			potentialLabel = new JLabel();
			forceLabel = new JLabel();
			auxPanel = new JPanel();
			this.addKeyListener(this);
			auxPanel.setLayout(new GridBagLayout());
			forceLabel.setText("Add charge to calculate force");
			potentialLabel.setText("Move mouse to calculate potential");
		}

		return auxPanel;
	}

	private JPanel getBottomPanel() {
		if (bottomPanel == null) {
			bottomPanel = new JPanel();
			bottomPanel.addKeyListener(this);
			bottomPanel.setLayout(new BorderLayout());
			bottomPanel.add(getClearAndReDrawPanel(), BorderLayout.EAST);
			bottomPanel.add(getAuxPanel(), BorderLayout.NORTH);
			bottomPanel.add(getStatusPanel(), BorderLayout.SOUTH);
			bottomPanel.add(getNumLandResNumPanel(), BorderLayout.WEST);
		}

		return bottomPanel;
	}

	private JPanel getButtonPanel() {
		if (buttonPanel == null) {
			buttonPanel = new JPanel();
			buttonPanel.setName("Button Panel");
			buttonPanel.setPreferredSize(new java.awt.Dimension(313, 136));
			buttonPanel.add(getAddChargeButton(), null);
			buttonPanel.add(getEditButton(), null);
			buttonPanel.add(getRemoveButton(), null);
		}

		return buttonPanel;
	}

	private JList getChargeList() {
		if (chargeList == null) {
			chargeList = new JList(Launcher.dat.getChargeArray());
			chargeList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			chargeList
					.addListSelectionListener(new javax.swing.event.ListSelectionListener() {

						public void valueChanged(
								javax.swing.event.ListSelectionEvent e) {
							if (currentCharge != null) {
								currentCharge.isSel = false;
							}
							int index = chargeList.getSelectedIndex();
							if ((index > 0)
									&& (index < Launcher.dat.getChargeLength())) {
								currentCharge = Launcher.dat.getCharge(index);
								currentCharge.isSel = true;
							} else {
								// currentCharge = null;
							}
							EFPanel.repaint();
						}
					});
			chargeList.addMouseListener(new java.awt.event.MouseAdapter() {

				@Override
				public void mouseClicked(java.awt.event.MouseEvent e) {
					if (e.getButton() != 1) {
						popUpMenu.show(chargeList, e.getX(), getY());
					}
				}
			});
		}

		return chargeList;
	}

	private JPanel getClearAndReDrawPanel() {
		if (clearAndReDrawPanel == null) {
			clearAndReDrawPanel = new JPanel();
			clearAndReDrawPanel.add(getClearButton(), null);
			clearAndReDrawPanel.add(getReDrawButton(), null);
		}

		return clearAndReDrawPanel;
	}

	private JButton getClearButton() {
		if (clearButton == null) {
			clearButton = new JButton("Clear Screen");
			clearButton.setActionCommand("CLEAR");
			clearButton.addActionListener(mwah);
		}

		return clearButton;
	}

	private JPanel getControls() {
		if (controls == null) {
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.fill = java.awt.GridBagConstraints.BOTH;
			gridBagConstraints2.gridy = 2;
			gridBagConstraints2.weightx = 1.0;
			gridBagConstraints2.weighty = 1.0;
			gridBagConstraints2.gridx = 0;

			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.gridx = 0;
			gridBagConstraints1.gridy = 1;

			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.gridy = 0;
			controls = new JPanel();
			this.addKeyListener(this);
			controls.setLayout(new GridBagLayout());
			controls.setPreferredSize(new java.awt.Dimension(208, 203));
			controls.setMinimumSize(new java.awt.Dimension(130, 203));
			controls.add(getButtonPanel(), gridBagConstraints);
			controls.add(getPaintModePanel(), gridBagConstraints1);
			controls.add(getListScrollPane(), gridBagConstraints2);
		}

		return controls;
	}

	private JButton getEditButton() {
		if (editButton == null) {
			editButton = new JButton("Edit");
			editButton.setActionCommand("EDIT");
			editButton.addActionListener(mwah);
		}

		return editButton;
	}

	private EFPanel getEFPanel() {
		if (EFPanel == null) {
			EFPanel = new EFPanel();
			EFPanel.addComponentListener(new java.awt.event.ComponentAdapter() {

				@Override
				public void componentResized(java.awt.event.ComponentEvent e) {
					EFPanel.resetSize();
					reDraw();
				}
			});
			EFPanel.add(this.popAltMenu);
			EFPanel.add(this.popUpMenu);

			EFPanel.addMouseListener(new MouseAdapter() {

				@Override
				public void mouseClicked(MouseEvent e) {
					boolean temp = checkSel(e);
					if (e.getButton() != 1) {
						if (temp) {
							popUpMenu.show(EFPanel, e.getX(), e.getY());
						} else {
							x2 = e.getX();
							y2 = e.getY();
							popAltMenu.show(EFPanel, e.getX(), e.getY());
						}
					}
					EFPanel.repaint();
				}

				@Override
				public void mouseReleased(MouseEvent arg0) {
					updateLabel();
					EFPanel.repaint();
					reDraw();
				}
			});
			EFPanel.addMouseMotionListener(new MouseMotionAdapter() {

				@Override
				public void mouseDragged(MouseEvent e) {
					checkSel(e);
					if (e.getButton() != 0) {
						return;
					}
					chargeMove(e);
					updateList();
				}

				@Override
				public void mouseMoved(MouseEvent e) {
					double[] data = Calc.totalForceAngle(e.getX(), e.getY());
					String force = "" + Calc.toTruncString(data[0], 2);
					String degree = "" + data[1];

					try {
						degree = degree.substring(0, 5);
					} catch (RuntimeException e1) {
						degree = degree.substring(0, 3);
					}

					String temp = force + " Newtons @ " + degree + "";
					forceLabel.setText(temp);

					String pot = Calc.toTruncString(Calc.totalPE(e.getX(), e
							.getY()), 2);
					potentialLabel.setText(pot + " Volts");

					if (currentCharge == null) {
						distLabel.setText("");
					} else {
						if (currentCharge != null) {
							double dist = Calc.dist(e.getX(), e.getY(),
									currentCharge.getX(), currentCharge.getY());
							dist /= Launcher.dat.siFactor;
							dist /= Launcher.dat.getScale();
							distLabel.setText(" Distance: "
									+ Calc.toTruncString(dist, 2) + "m");
						}
					}
				}
			});
		}

		return EFPanel;
	}

	private JCheckBox getFieldLineBox() {
		if (fieldLineBox == null) {
			fieldLineBox = new JCheckBox("Field Lines");
			fieldLineBox.setMinimumSize(new java.awt.Dimension(70, 23));
			fieldLineBox.setSelected(Launcher.dat.fieldLinesOn);
			fieldLineBox.addItemListener(new java.awt.event.ItemListener() {

				public void itemStateChanged(java.awt.event.ItemEvent e) {
					Launcher.dat.fieldLinesOn = fieldLineBox.isSelected();
					reDraw();
				}
			});
		}

		return fieldLineBox;
	}

	private JSlider getFieldLineSlider() {
		if (fieldLineSlider == null) {
			fieldLineSlider = new JSlider();
			fieldLineSlider.setMinimum(0);
			fieldLineSlider.setPaintTicks(true);
			fieldLineSlider.setPaintLabels(true);
			fieldLineSlider
					.setToolTipText("Move slider to set total number of Field Lines.");
			fieldLineSlider.setValue(Launcher.dat.getNumLines());
			fieldLineSlider.setMajorTickSpacing(20);
			fieldLineSlider.setMinorTickSpacing(5);
			fieldLineSlider
					.addChangeListener(new javax.swing.event.ChangeListener() {

						public void stateChanged(javax.swing.event.ChangeEvent e) {
							Launcher.dat
									.setNumLines(fieldLineSlider.getValue());
							reDraw();
						}
					});
		}

		return fieldLineSlider;
	}

	private JScrollPane getListScrollPane() {
		if (listScrollPane == null) {
			listScrollPane = new JScrollPane();
			listScrollPane.setAutoscrolls(true);
			listScrollPane.setViewportView(getChargeList());
		}

		return listScrollPane;
	}

	private JPanel getMainPanel() {
		if (mainPanel == null) {
			label = new JLabel();
			mainPanel = new JPanel();
			mainPanel.setLayout(new BorderLayout());
			label.setText("Loading...");
			mainPanel.add(getControls(), java.awt.BorderLayout.EAST);
			mainPanel.add(label, java.awt.BorderLayout.NORTH);
			mainPanel.add(getBottomPanel(), java.awt.BorderLayout.SOUTH);
			mainPanel.add(getEFPanel(), java.awt.BorderLayout.CENTER);
			mainPanel.addKeyListener(this);
		}

		return mainPanel;
	}

	private JMenuItem getMenuActionsRemoveAll() {
		if (menuActionsRemoveAll == null) {
		}

		return menuActionsRemoveAll;
	}

	private JPanel getNumLandResNumPanel() {
		if (numLandResNumPanel == null) {
			potRes = new JLabel();
			potRes.setText("Potential resolution:");
			numLinesLabel = new JLabel();
			numLinesLabel.setText("Field Line Density");
			numLandResNumPanel = new JPanel();
			numLandResNumPanel.add(numLinesLabel, null);
			numLandResNumPanel.add(getFieldLineSlider(), null);
			numLandResNumPanel.add(potRes, null);
			numLandResNumPanel.add(getPotResSlider(), null);
		}

		return numLandResNumPanel;
	}

	private JPanel getPaintModePanel() {
		if (paintModePanel == null) {
			paintModePanel = new JPanel();
			paintModePanel.setMinimumSize(new java.awt.Dimension(300, 60));
			paintModePanel.setPreferredSize(new java.awt.Dimension(196, 13));
			paintModePanel.add(getFieldLineBox(), null);
			paintModePanel.add(getPotLinesBox(), null);
		}

		return paintModePanel;
	}

	private JCheckBox getPotLinesBox() {
		if (potLinesBox == null) {
			potLinesBox = new JCheckBox("Potential Lines");
			potLinesBox.setSelected(Launcher.dat.potentialLinesOn);
			potLinesBox.addItemListener(new java.awt.event.ItemListener() {

				public void itemStateChanged(java.awt.event.ItemEvent e) {
					Launcher.dat.potentialLinesOn = potLinesBox.isSelected();
					reDraw();
				}
			});
		}

		return potLinesBox;
	}

	private JSlider getPotResSlider() {
		if (potResSlider == null) {
			potResSlider = new JSlider();
			potResSlider.setSize(new java.awt.Dimension(198, 56));
			potResSlider.setMajorTickSpacing(3);
			potResSlider.setMaximum(16);
			potResSlider.setMinimum(1);
			potResSlider.setPaintLabels(true);
			potResSlider.setPaintTicks(true);
			potResSlider.setPaintTrack(true);
			potResSlider
					.setToolTipText("Change to increase Potential line resolution");
			potResSlider.setValue(Launcher.dat.getPeRes());
			potResSlider.setInverted(false);
			potResSlider
					.addChangeListener(new javax.swing.event.ChangeListener() {

						public void stateChanged(javax.swing.event.ChangeEvent e) {
							Launcher.dat.setPeRes(potResSlider.getValue());
							reDraw();
						}
					});
		}

		return potResSlider;
	}

	private JButton getReDrawButton() {
		if (reDrawButton == null) {
			reDrawButton = new JButton("Redraw Screen");
			reDrawButton.setActionCommand("REDRAW");
			reDrawButton.addActionListener(mwah);
		}

		return reDrawButton;
	}

	private JButton getRemoveButton() {
		if (removeButton == null) {
			removeButton = new JButton("Remove");
			removeButton.setActionCommand("REMOVE");
			removeButton.addActionListener(mwah);
		}

		return removeButton;
	}

	private JPanel getStatusPanel() {
		if (statusPanel == null) {
			distLabel = new JLabel();
			statusPanel = new JPanel();
			statusPanel.addKeyListener(this);
			statusPanel.setLayout(new BorderLayout());
			distLabel.setText("Add charge to calculate distance");
			statusPanel.add(forceLabel, java.awt.BorderLayout.WEST);
			statusPanel.add(potentialLabel, java.awt.BorderLayout.EAST);
			statusPanel.add(distLabel, java.awt.BorderLayout.CENTER);
		}

		return statusPanel;
	}

	private void initialize() {
		MetalLookAndFeel mlaf = new MetalLookAndFeel();
		try {
			UIManager.setLookAndFeel(mlaf);
		} catch (UnsupportedLookAndFeelException e1) {

		}
		this.backColor = Launcher.dat.backgroundColor;
		this.imageURL = MainWindow.class.getResource("ef.png");
		this.setIconImage(Toolkit.getDefaultToolkit().createImage(imageURL));
		this.setName("Main Window");
		this.setTitle("Electric Field SIM - " + Launcher.version);

		if (Launcher.isApplet) {
			this.setTitle(this.getTitle() + " - APPLET");
		}

		this.addKeyListener(this);
		this.setPreferredSize(new java.awt.Dimension(Launcher.dat.mainWinWidth,
				Launcher.dat.MainWinHeight));
		this.setMinimumSize(new java.awt.Dimension(Launcher.dat.mainWinWidth,
				Launcher.dat.MainWinHeight));
		this.setSize(Launcher.dat.mainWinWidth, Launcher.dat.MainWinHeight);
		this.addComponentListener(new ComponentAdapter() {

			@Override
			public void componentResized(ComponentEvent e) {
				EFPanel.resetSize();
				if (getHeight() < 300) {
					setSize(300, getWidth());
				}
				if (getWidth() < 500) {
					setSize(getHeight(), 500);
				}
				Launcher.dat.MainWinHeight = getHeight();
				Launcher.dat.mainWinWidth = getWidth();
			}
		});
		this.addWindowListener(new WindowAdapter() {

			@Override
			public void windowClosing(WindowEvent e) {
				Launcher.Exit(0);
			}
		});
		this.setJMenuBar(setupJMenu());
		this.setupPopUpJMenu();
		this.add(getMainPanel());
		this.updateList();
	}

	private JMenuBar setupJMenu() {
		menuBar = new JMenuBar();
		menuFile = new JMenu("File");
		menuFileExit = new JMenuItem("Exit");
		menuFileExit.setActionCommand("EXIT");
		menuFileExit.addActionListener(mwah);
		menuFileOptions = new JMenuItem("Options...");
		menuFileOptions.setActionCommand("OPTIONS");
		menuFileOptions.addActionListener(mwah);
		menuFileLoad = new JMenuItem("Load...");
		menuFileLoad.setActionCommand("LOAD");
		menuFileLoad.addActionListener(mwah);
		menuFilePageSetup = new JMenuItem("Page Setup...");
		menuFilePageSetup.setActionCommand("PSETUP");
		menuFilePageSetup.addActionListener(mwah);
		menuFilePrint = new JMenuItem("Print...");
		menuFilePrint.setActionCommand("PRINT");
		menuFilePrint.addActionListener(mwah);
		menuFileSave = new JMenuItem("Save...");
		menuFileSave.setActionCommand("SAVE");
		menuFileSave.addActionListener(mwah);
		menuActions = new JMenu("Actions");
		menuActionsAdd = new JMenuItem("Add...");
		menuActionsAdd.setActionCommand("ADD");
		menuActionsAdd.addActionListener(mwah);
		menuActionsRemove = new JMenuItem("Remove");
		menuActionsRemove.setActionCommand("REMOVE");
		menuActionsRemove.addActionListener(mwah);
		menuActionsRemoveAll = new JMenuItem("Remove All");
		menuActionsRemoveAll.setActionCommand("REMOVE ALL");
		menuActionsRemoveAll.addActionListener(mwah);
		menuActionsEdit = new JMenuItem("Edit...");
		menuActionsEdit.setActionCommand("EDIT");
		menuActionsEdit.addActionListener(mwah);
		menuHelp = new JMenu("Help");
		menuHelpAbout = new JMenuItem("About...");
		menuHelpAbout.setActionCommand("ABOUT");
		menuHelpAbout.addActionListener(mwah);
		menuFile.add(menuFileOptions);
		menuFile.insertSeparator(2);
		menuFile.add(menuFileLoad);
		menuFile.add(menuFileSave);
		menuFile.insertSeparator(4);
		// menuFile.add(menuFilePageSetup); //these functions are not added
		// beacuse the underlying methods are not finished.
		// menuFile.add(menuFilePrint);
		// menuFile.insertSeparator(6);
		menuFile.add(menuFileExit);
		menuActions.add(menuActionsAdd);
		menuActions.add(menuActionsRemove);
		menuActions.add(getMenuActionsRemoveAll());
		menuActions.add(menuActionsEdit);
		menuHelp.add(menuHelpAbout);
		menuBar.add(menuFile);
		menuBar.add(menuActions);
		menuBar.add(menuHelp);

		if (Launcher.isApplet) {
			menuFileLoad.setEnabled(false);
			menuFileSave.setEnabled(false);
		}

		return menuBar;
	}

	private void setupPopUpJMenu() {
		popAddLine = new JMenuItem("Add Line Charge");
		popAddLine.setActionCommand("ADDLINEHERE");
		popAddLine.addActionListener(mwah);
		popAddLine.setEnabled(false);
		popAddPoint = new JMenuItem("Add Point Charge");
		popAddPoint.setActionCommand("ADDPOINTHERE");
		popAddPoint.addActionListener(mwah);
		popDrawPE = new JMenuItem("Draw Potential");
		popDrawPE.setActionCommand("DRAWHERE");
		popDrawPE.addActionListener(mwah);
		popDrawPE2 = new JMenuItem("Draw Potential...");
		popDrawPE2.setActionCommand("DRAWHERE...");
		popDrawPE2.addActionListener(mwah);
		popEdit = new JMenuItem("Edit...");
		popEdit.setActionCommand("EDIT");
		popEdit.addActionListener(mwah);
		popRemove = new JMenuItem("Remove");
		popRemove.setActionCommand("REMOVE");
		popRemove.addActionListener(mwah);
		popUpMenu = new JPopupMenu("PopUpJMenu");
		popUpMenu.add(popEdit);
		popUpMenu.add(popRemove);
		popAltMenu = new JPopupMenu("Alt PopUpJMenu");
		popAltMenu.add(this.popAddPoint);
		popAltMenu.add(this.popAddLine);
		popAltMenu.addSeparator();
		popAltMenu.add(this.popDrawPE);
		popAltMenu.add(this.popDrawPE2);
	}

	private void updateLabel() {
		if (currentCharge != null) {
			label.setText(currentCharge.toString());
		}
	}

	protected void nextCurrent() {
		for (Charge temp : Launcher.dat.getCharges()) {
			if (temp != null) {
				currentCharge = temp;

				return;
			}
		}
	}

	protected void updateList() {
		chargeList.setListData(Launcher.dat.getChargeArray());
	}
}
