/*
 * This file is part of the e-field program written by Adam Cornett
 * This program is distrubuted under the General Public License.
 * The license is distrubed with the source in the file 'gpl.txt'
 */
package com.ajc.esim.gui;

import java.awt.BorderLayout;
import java.awt.Choice;
import java.awt.Color;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Label;
import java.awt.event.ActionEvent;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.ajc.esim.core.Data;
import com.ajc.esim.core.Launcher;

/**
 * Program Options Dialog
 */
public class OptionsFrame extends EFDialog {

	public Color background = null;

	public Color neg = null;

	public Color poz = null;

	private JLabel backgroundColorLabel = null, gradMaxLabel = null,
			gradMinLabel = null, gridOnOffLabel = null,
			gridSpacingLabel = null, negChargeColorLabel = null,
			numPotLinesLabel = null, posChargeLabel = null,
			potLineLabel = null, potOffsetLabel = null, potResLabel = null,
			potSpacingLabel = null, prefixLabel = null, pxPerMeterLabel = null,
			textColorLabel = null, tickMarkLabel = null;

	private Label bgColor = null, negCharge = null, posCharge = null,
			potColor = null, textColor = null;

	private JButton button = null, button1 = null, button3 = null,
			button4 = null, button5 = null, cancelButton = null,
			okButton = null, reset = null;

	private Choice choice = null;

	private JPanel colorPanel = null, okPanel = null, optionsPanel = null;

	private Data data = Launcher.dat;

	private JTextField gradientMax = null, gradientMin = null,
			gridSpace = null, pLines = null, pOff = null, pSpace = null,
			potResField = null, siConv = null;

	private JCheckBox gridOnOff = null, tickOnOff = null;

	private Frame owner = null;

	/**
	 * Creates a new OptionsFrame object.
	 * 
	 */
	public OptionsFrame(JFrame owner) {
		super(owner);
		initialize();
		this.owner = owner;
		this.setLocationRelativeTo(null);
	}

	@Override
	public void actionPerformed(ActionEvent arg0) {
		updateColors();
		String temp = arg0.getActionCommand();
		if (temp.equals("OK")) {
			okAction();
		} else if (temp.equals("CANCEL")) {
			cancelAction();
		} else if (temp.length() > 9) {
			if (temp.substring(0, 9).equals("PICKCOLOR")) {
				if (temp.substring(9).equals("B")) {
					ColorChooser win = new ColorChooser(this.owner);
					win.jColorChooser.setColor(data.backgroundColor);
					win.sel = 1;
					win.setVisible(true);
				} else if (temp.substring(9).equals("P")) {
					ColorChooser win = new ColorChooser(this.owner);
					win.jColorChooser.setColor(data.posColor);
					win.sel = 2;
					win.setVisible(true);
				} else if (temp.substring(9).equals("N")) {
					ColorChooser win = new ColorChooser(this.owner);
					win.jColorChooser.setColor(data.negColor);
					win.sel = 3;
					win.setVisible(true);
				} else if (temp.substring(9).equals("T")) {
					ColorChooser win = new ColorChooser(this.owner);
					win.jColorChooser.setColor(data.text);
					win.sel = 4;
					win.setVisible(true);
				} else if (temp.substring(9).equals("O")) {
					ColorChooser win = new ColorChooser(this.owner);
					win.jColorChooser.setColor(data.potentialColor);
					win.sel = 5;
					win.setVisible(true);
				}
			}
		} else if (temp.equals("RESET")) {
			data.resetAll();
			this.setVars();
		}
		updateColors();
	}

	public void initialize() {
		this.setLayout(new BorderLayout());
		this.setTitle("Options");
		this.setSize(328, 475);
		this.setModal(true);
		this.add(getColorPanel(), java.awt.BorderLayout.NORTH);
		this.add(getOkPanel(), java.awt.BorderLayout.SOUTH);
		this.add(getOptionsPanel(), java.awt.BorderLayout.CENTER);
		this.addKeyListener(this);
		this.addFocusListener(new java.awt.event.FocusAdapter() {

			@Override
			public void focusGained(java.awt.event.FocusEvent e) {
				updateColors();
			}
		});
		this.addWindowListener(new java.awt.event.WindowAdapter() {

			@Override
			public void windowClosing(java.awt.event.WindowEvent e) {
				dispose();
			}
		});
		this.setVars();
	}

	public void setVars() {
		this.gradientMax.setText("" + data.gradMax);
		this.gradientMin.setText("" + data.gradMin);
		this.potResField.setText("" + data.getPeRes());
		this.pLines.setText("" + data.pLines);
		this.pOff.setText("" + data.potOffset);
		this.pSpace.setText("" + data.potspacing);
		this.siConv.setText("" + data.siFactor);
		this.gridOnOff.setSelected(data.gridOn);
		this.tickOnOff.setSelected(data.ticksOn);
		this.gridSpace.setText("" + data.tickSpacing);
		background = data.backgroundColor;
		poz = data.posColor;
		neg = data.negColor;
		this.updateColors();
	}

	public void updateColors() {
		potColor.setBackground(data.potentialColor);
		potColor.setForeground(data.potentialColor);
		bgColor.setBackground(data.backgroundColor);
		bgColor.setForeground(data.backgroundColor);
		posCharge.setBackground(data.posColor);
		posCharge.setForeground(data.posColor);
		negCharge.setBackground(data.negColor);
		negCharge.setForeground(data.negColor);
		textColor.setBackground(data.text);
		textColor.setForeground(data.text);
		potColor.repaint();
		textColor.repaint();
		bgColor.repaint();
		posCharge.repaint();
		negCharge.repaint();
	}

	private JButton getButton() {
		if (button == null) {
			button = new JButton("Change");
			button.setName("Background Color Change JButton");
			button.setActionCommand("PICKCOLORB"); // Generated
			button.addActionListener(this);
		}
		return button;
	}

	private JButton getButton1() {
		if (button1 == null) {
			button1 = new JButton("Change");
			button1.setName("Potential Line Color Change JButton");
			button1.setActionCommand("PICKCOLORO");
			button1.addActionListener(this);
		}
		return button1;
	}

	private JButton getButton3() {
		if (button3 == null) {
			button3 = new JButton("Change");
			button3.setName("Positive Charge Color Change JButton");
			button3.setActionCommand("PICKCOLORP"); // Generated
			button3.addActionListener(this);
		}
		return button3;
	}

	private JButton getButton4() {
		if (button4 == null) {
			button4 = new JButton("Change");
			button4.setName("Negtive Charge Color Change JButton");
			button4.setActionCommand("PICKCOLORN"); // Generated
			button4.addActionListener(this);
		}
		return button4;
	}

	private JButton getButton5() {
		if (button5 == null) {
			button5 = new JButton("Change");
			button5.setName("Charge Text Color Change JButton");
			button5.setActionCommand("PICKCOLORT"); // Generated
			button5.addActionListener(this);
		}
		return button5;
	}

	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton("Cancel");
			cancelButton.setActionCommand("CANCEL"); // Generated
			cancelButton.addActionListener(this);
		}
		return cancelButton;
	}

	private Choice getChoice() {
		if (choice == null) {
			choice = new Choice();
			choice.setEnabled(false);
			choice.addItemListener(new java.awt.event.ItemListener() {

				public void itemStateChanged(java.awt.event.ItemEvent e) {
					data.setPrefix(choice.getSelectedIndex());
				}
			});
			for (int i = 0; i < 9; i++) {
				choice.add(Data.prefixes[i].toString());
			}

			choice.select(data.getPrefixIndex());
		}
		return choice;
	}

	private JPanel getColorPanel() {
		if (colorPanel == null) {
			String colorLabel = "#########";
			potColor = new Label(colorLabel);
			bgColor = new Label(colorLabel);
			posCharge = new Label(colorLabel);
			negCharge = new Label(colorLabel);
			textColor = new Label(colorLabel);
			potLineLabel = new JLabel("Potential Line");
			textColorLabel = new JLabel("Text:");
			negChargeColorLabel = new JLabel("Negtive Charge:");
			posChargeLabel = new JLabel("Postive Charge:");
			backgroundColorLabel = new JLabel("Background:");
			colorPanel = new JPanel();
			colorPanel.setLayout(new GridBagLayout());

			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints31 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints10 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints12 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints13 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints14 = new GridBagConstraints();
			gridBagConstraints3.gridx = 0;
			gridBagConstraints3.gridy = 1;
			gridBagConstraints4.gridx = 2;
			gridBagConstraints4.gridy = 1;
			gridBagConstraints5.gridx = 1;
			gridBagConstraints5.gridy = 1;
			gridBagConstraints6.gridx = 0;
			gridBagConstraints6.gridy = 2;
			gridBagConstraints7.gridx = 1;
			gridBagConstraints7.gridy = 2;
			gridBagConstraints8.gridx = 2;
			gridBagConstraints8.gridy = 2;
			gridBagConstraints9.gridx = 0;
			gridBagConstraints9.gridy = 3;
			gridBagConstraints10.gridx = 1;
			gridBagConstraints10.gridy = 3;
			gridBagConstraints12.gridx = 0;
			gridBagConstraints12.gridy = 4;
			gridBagConstraints13.gridx = 1;
			gridBagConstraints13.gridy = 4;
			gridBagConstraints5.insets = new java.awt.Insets(1, 2, 1, 2);
			gridBagConstraints7.insets = new java.awt.Insets(1, 2, 1, 2);
			gridBagConstraints10.insets = new java.awt.Insets(1, 2, 1, 2);
			gridBagConstraints3.anchor = java.awt.GridBagConstraints.EAST;
			gridBagConstraints6.anchor = java.awt.GridBagConstraints.EAST;
			gridBagConstraints9.anchor = java.awt.GridBagConstraints.EAST;
			gridBagConstraints12.anchor = java.awt.GridBagConstraints.EAST;
			gridBagConstraints13.insets = new java.awt.Insets(1, 2, 1, 2);
			gridBagConstraints14.gridx = 2;
			gridBagConstraints14.gridy = 4;
			gridBagConstraints11.gridx = 2;
			gridBagConstraints11.gridy = 3;
			gridBagConstraints1.gridx = 2;
			gridBagConstraints1.gridy = 5;
			gridBagConstraints1.insets = new java.awt.Insets(0, 0, 1, 0);
			gridBagConstraints2.gridx = 1;
			gridBagConstraints2.gridy = 5;
			gridBagConstraints2.insets = new java.awt.Insets(0, 0, 1, 0);
			gridBagConstraints31.gridx = 0;
			gridBagConstraints31.gridy = 5;
			gridBagConstraints31.anchor = java.awt.GridBagConstraints.EAST;
			gridBagConstraints31.insets = new java.awt.Insets(0, 0, 1, 0);

			colorPanel.add(bgColor, gridBagConstraints5);
			colorPanel.add(posCharge, gridBagConstraints7);
			colorPanel.add(negCharge, gridBagConstraints10);
			colorPanel.add(textColor, gridBagConstraints13);
			colorPanel.add(backgroundColorLabel, gridBagConstraints3);
			colorPanel.add(posChargeLabel, gridBagConstraints6);
			colorPanel.add(negChargeColorLabel, gridBagConstraints9);
			colorPanel.add(textColorLabel, gridBagConstraints12);
			colorPanel.add(getButton(), gridBagConstraints4);
			colorPanel.add(getButton3(), gridBagConstraints8);
			colorPanel.add(getButton5(), gridBagConstraints14);
			colorPanel.add(getButton4(), gridBagConstraints11);
			colorPanel.add(getButton1(), gridBagConstraints1);
			colorPanel.add(potColor, gridBagConstraints2);
			colorPanel.add(potLineLabel, gridBagConstraints31);
			colorPanel.addKeyListener(this);
		}
		return colorPanel;
	}

	private JTextField getGradientMax() {
		if (gradientMax == null) {
			gradientMax = new JTextField();
			gradientMax.addKeyListener(this);
			gradientMax.setInputVerifier(new DoubleVerifier());
		}
		return gradientMax;
	}

	private JTextField getGradientMin() {
		if (gradientMin == null) {
			gradientMin = new JTextField();
			gradientMin.addKeyListener(this);
			gradientMin.setInputVerifier(new DoubleVerifier());
		}
		return gradientMin;
	}

	private JCheckBox getGridOnOff() {
		if (gridOnOff == null) {
			gridOnOff = new JCheckBox();
		}
		return gridOnOff;
	}

	private JTextField getGridSpace() {
		if (gridSpace == null) {
			gridSpace = new JTextField();
			gridSpace.setInputVerifier(new DoubleVerifier());
		}
		return gridSpace;
	}

	private JButton getOkButton() {
		if (okButton == null) {
			okButton = new JButton("OK");
			okButton.addActionListener(this);
		}
		return okButton;
	}

	private JPanel getOkPanel() {
		if (okPanel == null) {
			okPanel = new JPanel();
			okPanel.add(getOkButton(), null);
			okPanel.add(getCancelButton(), null);
			okPanel.add(getReset(), null);
		}
		return okPanel;
	}

	private JPanel getOptionsPanel() {
		if (optionsPanel == null) {
			/*
			 * Initalize Labels
			 */
			potResLabel = new JLabel("Potential Resolution:");
			gradMinLabel = new JLabel("Gradient Min");
			tickMarkLabel = new JLabel("Tick Marks on/off");
			prefixLabel = new JLabel("Charge Prefix:");
			gridSpacingLabel = new JLabel("Grid Spacing (Pixels)");
			pxPerMeterLabel = new JLabel("Pixels per meter");
			potOffsetLabel = new JLabel("Potential Offset (V)");
			potSpacingLabel = new JLabel("Potential Spacing (V)");
			gridOnOffLabel = new JLabel("Grid on/off");
			numPotLinesLabel = new JLabel("# Potential Lines");
			gradMaxLabel = new JLabel("Gradient Max");
			optionsPanel = new JPanel();
			optionsPanel.setLayout(new GridBagLayout());

			GridBagConstraints potResFCon = new GridBagConstraints();
			potResFCon.fill = GridBagConstraints.HORIZONTAL;
			potResFCon.gridy = 0;
			potResFCon.weightx = 1.0;
			potResFCon.gridx = 2;

			GridBagConstraints choiceCon = new GridBagConstraints();
			choiceCon.gridx = 2;
			choiceCon.gridy = 11;

			GridBagConstraints potSpaceLCon = new GridBagConstraints();
			potSpaceLCon.gridx = 1;
			potSpaceLCon.gridy = 5;
			potSpaceLCon.anchor = GridBagConstraints.EAST;
			potSpaceLCon.insets = new java.awt.Insets(0, 0, 1, 0);

			GridBagConstraints tickLCon = new GridBagConstraints();
			tickLCon.gridx = 1;
			tickLCon.gridy = 10;
			tickLCon.anchor = GridBagConstraints.EAST;

			GridBagConstraints tickCheckCon = new GridBagConstraints();
			tickCheckCon.gridx = 2;
			tickCheckCon.gridy = 10;
			tickCheckCon.anchor = GridBagConstraints.WEST;

			GridBagConstraints prefixLCon = new GridBagConstraints();
			prefixLCon.gridx = 1;
			prefixLCon.gridy = 11;

			GridBagConstraints gridSpaceLCon = new GridBagConstraints();
			gridSpaceLCon.gridx = 1;
			gridSpaceLCon.gridy = 8;

			GridBagConstraints gridSpaceFCon = new GridBagConstraints();
			gridSpaceFCon.gridx = 2;
			gridSpaceFCon.gridy = 8;
			gridSpaceFCon.weightx = 1.0;
			gridSpaceFCon.fill = GridBagConstraints.HORIZONTAL;

			GridBagConstraints potoffsetLCon = new GridBagConstraints();
			potoffsetLCon.gridx = 1;
			potoffsetLCon.gridy = 6;
			potoffsetLCon.anchor = GridBagConstraints.EAST;
			potoffsetLCon.insets = new java.awt.Insets(0, 0, 1, 0);

			GridBagConstraints pxPMLCon = new GridBagConstraints();
			pxPMLCon.gridx = 1;
			pxPMLCon.gridy = 7;
			pxPMLCon.anchor = GridBagConstraints.EAST;
			pxPMLCon.insets = new java.awt.Insets(0, 0, 1, 0);

			GridBagConstraints pOffFCon = new GridBagConstraints();
			pOffFCon.gridx = 2;
			pOffFCon.gridy = 6;
			pOffFCon.weightx = 1.0;
			pOffFCon.fill = GridBagConstraints.HORIZONTAL;

			GridBagConstraints siConvCon = new GridBagConstraints();
			siConvCon.gridx = 2;
			siConvCon.gridy = 7;
			siConvCon.weightx = 1.0;
			siConvCon.fill = GridBagConstraints.HORIZONTAL;

			GridBagConstraints pSpaceCon = new GridBagConstraints();
			pSpaceCon.gridx = 2;
			pSpaceCon.gridy = 5;
			pSpaceCon.weightx = 1.0;
			pSpaceCon.fill = GridBagConstraints.HORIZONTAL;

			GridBagConstraints gridOFLCon = new GridBagConstraints();
			gridOFLCon.gridx = 1;
			gridOFLCon.gridy = 9;
			gridOFLCon.anchor = GridBagConstraints.EAST;
			gridOFLCon.insets = new java.awt.Insets(0, 0, 1, 0);

			GridBagConstraints gridOFCCon = new GridBagConstraints();
			gridOFCCon.gridx = 2;
			gridOFCCon.gridy = 9;
			gridOFCCon.anchor = GridBagConstraints.WEST;

			GridBagConstraints numPotLCon = new GridBagConstraints();
			numPotLCon.gridx = 1;
			numPotLCon.gridy = 3;
			numPotLCon.anchor = GridBagConstraints.EAST;
			numPotLCon.insets = new java.awt.Insets(0, 0, 1, 0);

			GridBagConstraints pLinesCon = new GridBagConstraints();
			pLinesCon.gridx = 2;
			pLinesCon.gridy = 3;
			pLinesCon.weightx = 1.0;
			pLinesCon.fill = GridBagConstraints.HORIZONTAL;

			GridBagConstraints gradMinFCon = new GridBagConstraints();
			gradMinFCon.gridx = 2;
			gradMinFCon.gridy = 1;
			gradMinFCon.weightx = 1.0;
			gradMinFCon.fill = GridBagConstraints.HORIZONTAL;

			GridBagConstraints gradMaxLCon = new GridBagConstraints();
			gradMaxLCon.gridx = 1;
			gradMaxLCon.gridy = 2;
			gradMaxLCon.anchor = java.awt.GridBagConstraints.EAST;
			gradMaxLCon.insets = new java.awt.Insets(0, 0, 1, 0);

			GridBagConstraints gradMaxFCon = new GridBagConstraints();
			gradMaxFCon.gridx = 2;
			gradMaxFCon.gridy = 2;
			gradMaxFCon.weightx = 1.0;
			gradMaxFCon.fill = GridBagConstraints.HORIZONTAL;

			GridBagConstraints potResCon = new GridBagConstraints();
			potResCon.gridx = 0;
			potResCon.gridy = 0;
			potResCon.gridwidth = 2;
			potResCon.anchor = GridBagConstraints.EAST;
			potResCon.insets = new java.awt.Insets(0, 0, 1, 0);

			GridBagConstraints gradMinLCon = new GridBagConstraints();
			gradMinLCon.gridx = 1;
			gradMinLCon.gridy = 1;
			gradMinLCon.anchor = GridBagConstraints.EAST;
			gradMinLCon.insets = new java.awt.Insets(0, 0, 1, 0);

			/*
			 * Add all elements to the Panel
			 */
			optionsPanel.add(potResLabel, potResCon);
			optionsPanel.add(gradMinLabel, gradMinLCon);
			optionsPanel.add(getGradientMin(), gradMinFCon);
			optionsPanel.add(gradMaxLabel, gradMaxLCon);
			optionsPanel.add(getGradientMax(), gradMaxFCon);
			optionsPanel.add(numPotLinesLabel, numPotLCon);
			optionsPanel.add(getPLines(), pLinesCon);
			optionsPanel.add(gridOnOffLabel, gridOFLCon);
			optionsPanel.add(getGridOnOff(), gridOFCCon);
			optionsPanel.add(potSpacingLabel, potSpaceLCon);
			optionsPanel.add(potOffsetLabel, potoffsetLCon);
			optionsPanel.add(pxPerMeterLabel, pxPMLCon);
			optionsPanel.add(getPOff(), pOffFCon);
			optionsPanel.add(getPSpace(), pSpaceCon);
			optionsPanel.add(getSiConv(), siConvCon);
			optionsPanel.add(gridSpacingLabel, gridSpaceLCon);
			optionsPanel.add(getGridSpace(), gridSpaceFCon);
			optionsPanel.add(getChoice(), choiceCon);
			optionsPanel.add(prefixLabel, prefixLCon);
			optionsPanel.add(tickMarkLabel, tickLCon);
			optionsPanel.add(getTickOnOff(), tickCheckCon);
			optionsPanel.add(getPotResField(), potResFCon);
		}
		return optionsPanel;
	}

	private JTextField getPLines() {
		if (pLines == null) {
			pLines = new JTextField();
			pLines.addKeyListener(this);
			pLines.setInputVerifier(new DoubleVerifier());
		}
		return pLines;
	}

	private JTextField getPOff() {
		if (pOff == null) {
			pOff = new JTextField();
			pOff.setInputVerifier(new DoubleVerifier());
		}
		return pOff;
	}

	private JTextField getPotResField() {
		if (potResField == null) {
			potResField = new JTextField();
			potResField.setInputVerifier(new DoubleVerifier());
		}
		return potResField;
	}

	private JTextField getPSpace() {
		if (pSpace == null) {
			pSpace = new JTextField();
			pSpace.setInputVerifier(new DoubleVerifier());
		}
		return pSpace;
	}

	private JButton getReset() {
		if (reset == null) {
			reset = new JButton("Reset All Settings");
			reset.setActionCommand("RESET");
			reset.addActionListener(this);
		}
		return reset;
	}

	private JTextField getSiConv() {
		if (siConv == null) {
			siConv = new JTextField();
			siConv.setInputVerifier(new DoubleVerifier());
		}
		return siConv;
	}

	private JCheckBox getTickOnOff() {
		if (tickOnOff == null) {
			tickOnOff = new JCheckBox();
		}
		return tickOnOff;
	}

	private int Sync() {
		int status = 0;
		data.gridOn = gridOnOff.isSelected();
		data.ticksOn = tickOnOff.isSelected();
		try {
			data.pLines = Integer.parseInt(pLines.getText());
			data.setPeRes(Integer.parseInt(potResField.getText()));
			data.gradMax = Double.parseDouble(gradientMax.getText());
			data.gradMin = Double.parseDouble(gradientMin.getText());
			data.potOffset = Long.parseLong(pOff.getText());
			data.potspacing = Long.parseLong(pSpace.getText());
			data.siFactor = Double.parseDouble(siConv.getText());
			data.tickSpacing = Integer.parseInt(gridSpace.getText());
		} catch (NumberFormatException nfe) {
			JOptionPane.showMessageDialog(this, nfe, "Number Format Error",
					JOptionPane.ERROR_MESSAGE);
			status = 1;
		}
		Launcher.win.updateList();
		return status;
	}

	@Override
	protected void cancelAction() {
		this.dispose();
	}

	@Override
	protected void okAction() {
		if (this.Sync() == 0)
			this.dispose();
	}
} // @jve:decl-index=0:visual-constraint="10,10"
